<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */
/**
 * Exception extenstion.
 */
class PHPDS_exception extends Exception
{
	protected $ignoreLines = 0;
	protected $extendedMessage = '';

	public function __construct($message = "", $code = 0, $previous = null)
	{
		if (is_a($previous, 'Exception')) {
			$this->file = $previous->getFile();
			$this->line = $previous->getLine();
			$this->message .= $previous->getMessage();
			$this->trace = $previous->getTrace();
			$this->code = $previous->getCode();
			if (empty($this->code)) $this->code = $code;
			if (is_a($previous, 'PHPDS_exception')) $this->extendedMessage .= $previous->getExtendedMessage();
			if ($message) $this->extendedMessage .= $message;
		} else {
			$this->message = $message;
			$this->code = $code;
		}

		/*$this->ignoreLines = $code;
		$this->code = 0;*/
	}

	public function getignoreLines()
	{
		return $this->ignoreLines;
	}

	public function getExtendedMessage()
	{
		return $this->extendedMessage;
	}

	public function getExtendedTrace()
	{
		return empty($this->trace) ? $this->getTrace() : $this->trace;
	}

	/**
	 * some Exception may choose to display some possible cause for the error, to help tracking down the error
	 */
	public function hasCauses()
	{
		return false;
	}

	/**
	 *  returns a special message and a list of possible causes
	 */
	public function getCauses()
	{
		return null;
	}
}

class PHPDS_fatalError extends PHPDS_exception
{
	public function __construct($message = "", $code = 0, $previous = null)
	{
		$error = error_get_last();

		if (isset($error['message'])) $this->message = $error['message'];
		if (isset($error['type'])) $this->code = $error['type'];
		if (isset($error['file'])) $this->file = $error['file'];
		if (isset($error['line'])) $this->line = $error['line'];

		//$this->ignoreLines = 2;
	}
}

class PHPDS_databaseException extends PHPDS_exception
{

	public function hasCauses()
	{
		return in_array($this->getCode(), array(
			1045, // access denied
			0,  // unknown error
			1049,  // unknown database
			2002,
			1146 // table doesn't exist
			));
	}

	public function getCauses()
	{
		switch ($this->getCode()) {
			case 1045: $special = 'db_access_denied'; break;
			case 0: $special = 'db_unknown'; break;
			case 1049: $special = 'db_unknown'; break;
			case 2002: $special = 'db_silent'; break;
			case 1146: $special = 'db_noexist'; break;
		}

		$phpds_not_installed = array(
			'<div class="warning">PHPDevShell have a database error, did you install yet?<div>',
			'If you didn\'t process the installation procedure, you should <a href="other/service/index.php">run the installer</a> now.'
		);
		$db_wrong_cred = array(
			'<div class="warning">Possibly wrong credentials have been given in the configuration file.</div>',
			'Please check the content of your configuration file(s).'
		);
		$db_wrong_dbname = array(
			'<div class="warning">Possibly wrong database name has been given in the configuration file.<div>',
			'Please check the content of your configuration file(s).'
		);
		$db_denies = array(
			'<div class="warning">The server wont accept the connection.</div>',
			'Please check that the database server is up and running.'
		);

		switch ($special) {
			case 'db_access_denied': $result = array(
				'Access to the database was not granted using the parameters set in the configuration file.',
				array($phpds_not_installed, $db_wrong_cred, $db_denies)
			); break;
			case 'db_silent': $result = array(
				'Unable to connect to the database.',
				array($db_wrong_cred, $db_denies)
			); break;
			case 'db_unknown': $result = array(
				'The connection to the server is ok but the database is unknown.',
				array($phpds_not_installed, $db_wrong_dbname)
			); break;
			case 'db_noexist': $result = array(
				'The connection to the server is ok and the database is known but the table doesn\'t exists.',
				array($phpds_not_installed, $db_wrong_dbname)
			); break;
			default: $result = array(
				'Unknown special case.',
				array()
			);
		}
		return $result;
	}

}


class PHPDS_securityException extends PHPDS_exception
{
	// Nothing required today.
}

class PHPDS_securityException403 extends PHPDS_exception
{
	// Nothing required today.
}

class PHPDS_pageException404 extends PHPDS_exception
{
	// Nothing required today.
}



class PHPDS_sprintfnException extends PHPDS_exception
{

	public function __construct($message = "", $code = 0, $previous = null) // CAUTION this declaration is NOT correct
	{
		$msg = '<p>The faulty string source is:<br /><pre class="ui-state-highlight ui-corner-all">'.htmlentities($message).'</pre><br />';
		if (!empty($code)) $msg .= 'The parameters were:<br /><tt>'.PU_dumpArray($code, null, true).'</tt></p>';

		parent::__construct($msg, 0, $previous);
	}

	public function hasCauses()
	{
		return true;
	}

	public function getCauses()
	{
		$result = array(
			'Unable to build a string with <i>sprintfn</i>',
			array(
				array('Some template or theme file has altered a module which does comply to the given parameters.', 'Try a different theme or check for possible typos in the theme module list')
			)
		);

		return $result;
	}

}
