<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 * This is the main engine.
 */
/**
 *
 */
class PHPDS_navigation extends navigation
{
	/**
	 * @var array
	 */
	protected $breadcrumbArray = null;
	/**
	 * @var string
	 */
	public $child = null;
	/**
	 * Holds all menu item information.
	 *
	 * @var array
	 */
	public $navigation;
	/**
	 * Holds all menu item information.
	 *
	 * @var array
	 */
	public $navAlias;

	/**
	 * This methods loads the menu structure, this according to permission and conditions.
	 *
	 * @return string
	 * @author Jason Schoeman
	 */
	public function extractMenu ()
	{
		$db = $this->db;
		// Get all users role ids.
		$all_user_roles = $this->user->getRoles($this->configuration['user_id']);
		// Get required menu data.
		if ($db->cacheEmpty('navigation')) {
			if (empty($this->navigation)) $this->navigation = array();
			if (empty($this->child)) $this->child = array();
			if (empty($this->navAlias)) $this->alias = array();
			$db->invokeQuery('NAVIGATION_extractMenuQuery', $all_user_roles);

			// Write menu data to cache.
			$db->cacheWrite('navigation', $this->navigation);
			$db->cacheWrite('child_navigation', $this->child);
			$db->cacheWrite('nav_alias', $this->navAlias);
		} else {
			$this->navigation = $db->cacheRead('navigation');
			$this->child = $db->cacheRead('child_navigation');
			$this->navAlias = $db->cacheRead('nav_alias');
		}

		return $this;
	}

		/**
	 * Determines what the menu item should be named.
	 *
	 * @param string $replacement_name
	 * @param string $menu_link
	 * @param int $menu_id
	 * @return string
	 */
	public function determineMenuName ($replacement_name = '', $menu_link = '', $menu_id = false, $plugin='')
	{
		$lang = $this->lang;
		// Do we have a language replacement for this item?
		if (! empty($replacement_name)) {
			return __("$replacement_name", "$plugin");
		} else if (! empty($lang[$menu_id])) {
			return $lang[$menu_id];
		} else if (! empty($lang[$menu_link])) {
			return $lang[$menu_link];
		} else {
			return $menu_link;
		}
	}

	/**
	 * Returns true if menu should show.
	 *
	 * @param integer $hide_type
	 * @param integer $menu_id
	 */
	public function showMenu ($hide_type, $menu_id = false)
	{
		// Check if we have a special condition.
		if (! empty($menu_id) && ($hide_type == 4) && $this->configuration['m'] == $menu_id) {
			return true;
		} else {
			// Check if we should show.
			if ($hide_type == 0 || $hide_type == 2) {
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	 * Compiles menu items in order.
	 *
	 * @return string
	 * @author Jason Schoeman
	 */
	public function createMenuStructure ()
	{
		// Define.
		$menu = false;
		$configuration = $this->configuration;
		$nav = $this->navigation;
		// Make sure array is not empty, otherwise error will be produced.
		if (! empty($nav)) {
			// Get menu group to compare against.
			if (! empty($nav[$configuration['m']]['is_parent'])) {
				$menu_group = $configuration['m'];
			} else {
				if (! empty($nav[$configuration['m']]['parent_menu_id'])) {
					$menu_group = $nav[$configuration['m']]['parent_menu_id'];
				} else {
					$menu_group = 0;
				}
			}
			// Start dynamic render engine.
			foreach ($nav as $currentkey => $m) {
				// Check if active item is parent.
				if ($this->showMenu($m['hide'], $m['menu_id']) && $nav[$m['menu_id']]['parent_menu_id'] == $menu_group) {

					// Check if current looped url is active.
					($m['menu_id'] == $configuration['m']) ? $url_active = 'current' : $url_active = 'inactive';
					// Swith menu types.
					if ($m['is_parent'] == 1) {
						// Root item with Children.
						$call_family = $this->callFamily($m['menu_id']);
						if (! empty($call_family)) {
							$call_family = '<ul class="navparent">' . $call_family . '</ul>';
							$p_type = 'grandparent';
						} else {
							$p_type = $url_active;
						}
						$menu .= "<li class=\"$p_type\">" . sprintf($m['url'], '<span class="nav-grand"></span>') . $call_family . '</li>';
					} else {
						// Root item with no Children.
						$menu .= "<li class=\"$url_active\">" . sprintf($m['url'], '<span class="child"></span>') . '</li>';
					}
				}
			}
			// If no menu item is present, at least call its parent!
			if (empty($menu)) {
				$menu = '<li class="current">' . sprintf($nav[$configuration['m']]['url'], '') . '</li>';
			}
		}
		// Return Menu.
		return $menu;

	}

	/**
	 * Assists write_menu in calling menu children.
	 *
	 * @param int $menu_id
	 */
	public function callFamily ($menu_id = false)
	{
		// Define.
		$menu = false;
		$configuration = $this->configuration;
		$nav = $this->navigation;
		// Only loop when we have something.
		if (! empty($this->child[$menu_id])) {
			// Loop menu items finding its children.
			$child = $this->child[$menu_id];
			foreach ($child as $currentkey => $m) {
				// Should item be shown.
				if ($this->showMenu($nav[$m]['hide'], $m)) {
					// Check if current looped url is active.
					($m == $configuration['m']) ? $url_active = 'current' : $url_active = 'inactive';
					// Check if we have a parent, if we do, we need to call her children.
					if ($nav[$m]['is_parent'] == 1) {
						// Child item with Children.
						$call_family = $this->callFamily($m);
						if (! empty($call_family)) {
							$call_family = '<ul class="ulchild">' . $call_family . '</ul>';
							$p_type = 'parent';
						} else {
							$p_type = $url_active;
						}
						$menu .= "<li class=\"$p_type\">" . sprintf($nav[$m]['url'], '<span class="nav-parent"></span>') . $call_family . '</li>';
					} else {
						// Child item without Children.
						$menu .= "<li class=\"$url_active\">" . sprintf($nav[$m]['url'], '<span class="child"></span>') . '</li>';
					}
				}
			}
		}
		return $menu;
	}

	/**
	 * This method compiles the history tree seen, this is the tree that the user sees expand when going deeper into menu levels.
	 * On the default template this is the navigation link string top left above the menus.
	 *
	 * @return string
	 */
	public function createBreadcrumbs ()
	{
		$configuration = $this->configuration;
		$nav = $this->navigation;
		// Check if user has permission to view Jump Menu!
		if (isset($nav['940041356']['menu_id'])) {
			$jump_menu = '<li class="jump"><a href="' . $this->buildURL(940041356) . '"><span></span>' . __('Dashboard', 'PHPDevShell') . '</a></li>';
		} else {
			$jump_menu = '';
		}
		// Front Page.
		$main_item = '<li class="home"><a href="' . $configuration['absolute_url'] . '/"><span></span>' . ___('Front Page') . '</a></li>' . $jump_menu;
		// Call breadcrumb method.
		$this->callbackParentItem($configuration['m']);
		// Set.
		$history_url = '';
		foreach (array_reverse($this->breadcrumbArray, true) as $key => $breadcrumb_id) {
			if (! empty($nav[$breadcrumb_id]['menu_name']) && (($breadcrumb_id != $configuration['m']) || ($key != 0))) {
				if ($nav[$breadcrumb_id]['is_parent']) {
					$bread_parent = $this->callFamily($breadcrumb_id);
					if (! empty($bread_parent))
						$bread_parent = '<ul class="breadparent">' . $bread_parent . '</ul>';
					$history_url .= '<li class="grandparent">' . sprintf($nav[$breadcrumb_id]['url'], '<span class="nav-grand"></span>') . $bread_parent . '</li>';
				} else {
					$history_url .= '</li>' . sprintf($nav[$breadcrumb_id]['url'], '') . '</a></li>';
				}
			}
		}
		// Go up one level.
		if (! empty($nav[$configuration['m']]['parent_menu_id'])) {
			$up_parent_id = $nav[$configuration['m']]['parent_menu_id'];
			if (! empty($nav[$up_parent_id]['type']) && $nav[$up_parent_id]['type'] > 3 ) {
				$up_parent_id = 0;
			}
		} else {
			$up_parent_id = 0;
		}
		// Create Up link.
		if (! empty($up_parent_id)) {
			// Create level up icon.
			$up = '<li class="up"><a href="' . $this->buildURL($up_parent_id) . '"><span></span>' . ___('Up') . '</a></li>';
		} else {
			$up = '';
		}
		// Do we have the history URL?
		if (empty($history_url)) $history_url = false;
		// Return items, in order.
		return $main_item . $history_url . $up;
	}

	/**
	 * Method assists method generate_history_tree in getting breadcrumb links.
	 *
	 * @param integer
	 * @return array
	 */
	private function callbackParentItem ($menu_id_)
	{
		$nav = $this->navigation;
		// Check what called menu items parent id is.
		if (! empty($nav[$menu_id_]['parent_menu_id'])) {
			$recall_parent_menu_id = $nav[$menu_id_]['parent_menu_id'];
		} else {
			$recall_parent_menu_id = 0;
		}
		// Return results.
		$this->breadcrumbArray[] = $menu_id_;
		// Call function itself to see if this item has a parent item.
		if ($recall_parent_menu_id != 0) {
			$this->callbackParentItem($recall_parent_menu_id);
		}
	}

	/**
	 * Simply returns current menu id.
	 *
	 * @return int
	 */
	public function currentMenuID()
	{
		return $this->configuration['m'];
	}

	/**
	 * Will try and locate the full path of a filename of a given menu id, if it is a link, the original filename will be returned.
	 *
	 * @param int $menu_id
	 * @param string $plugin
	 * @return string
	 */
	public function menuFile ($menu_id=false, $plugin=false)
	{
		if (empty($menu_id)) $menu_id = $this->configuration['m'];
		$absolute_path = $this->configuration['absolute_path'];
		list($plugin, $menu_link) = $this->menuPath($menu_id, $plugin);
		if (file_exists($absolute_path . 'plugins/' . $plugin . '/controllers/' . $menu_link)) {
			return $absolute_path . 'plugins/' . $plugin . '/controllers/' . $menu_link;
		} else if (file_exists($absolute_path . 'plugins/' . $plugin . '/' . $menu_link)) {
			return $absolute_path . 'plugins/' . $plugin . '/' . $menu_link;
		} else {
			return false;
		}
	}

	/**
	 * Will locate the menus item full path.
	 *
	 * @param int $menu_id
	 * @param string $plugin
	 * @return array
	 */
	public function menuPath ($menu_id=false, $plugin=false)
	{
		$configuration = $this->configuration;
		$navigation = $this->navigation;
		// Do we need a custom menu id link?
		if (empty($configuration['m']))
			$configuration['m'] = 0;
		if (empty($menu_id)) $menu_id =  $configuration['m'];
		// Obtain correct menu file.
		if (empty($navigation[$menu_id]['extend'])) {
			if (!empty($navigation[$menu_id])) {
				$menu_link = $navigation[$menu_id]['menu_link'];
				// Set plugin for this menu item.
				if (empty($plugin))
					$plugin = $navigation[$menu_id]['plugin'];
			}
		} else {
			$extend = $navigation[$menu_id]['extend'];
			$menu_link = $navigation[$extend]['menu_link'];
			// Set plugin for this menu item.
			if (empty($plugin))
				$plugin = $navigation[$extend]['plugin'];
		}
		if (empty($plugin))
			$plugin = 'PHPDevShell';
		if (empty($menu_link))
			$menu_link = '';
		// Create location path.
		return array($plugin, $menu_link);
	}

	/**
	 * Will return the url for a certain menu item when path is provided.
	 * @param string $item_path The string to the path of the menu item, 'user/control-panel.php'
	 * @param string $plugin_name The plugin name to look for it under, if empty, active plugin will be used.
	 * @param string $extend_url Will extend url with some get values.
	 * @return string Will return complete and cleaned sef url if available else normal url will be returned.
	 */
	public function buildURLFromPath ($item_path, $plugin_name = '', $extend_url = '')
	{
		// What is our developer looking for?
		if (empty($plugin_name))
			$plugin_name = $this->core->activePlugin();
		$lookup = array('plugin'=>$plugin_name, 'menu_link'=>$item_path);
		// Lets loop and look if we can find anything.
		$menu_id = PU_ArraySearch($lookup, $this->navigation);
		if (! empty($menu_id)) {
			return $this->buildURL($menu_id, $extend_url);
		} else {
			return $this->pageNotFound();
		}
	}

	/**
	 * Returns the correct string for use in href when creating a link for a menu id. Will return sef url if possible.
	 * Will return self url when no menu id is given. No starting & or ? is needed, this gets auto determined!
	 * If left empty it will return current active menu.
	 *
	 * @param mixed The menu id or menu file location to create a url from.
	 * @param string extend_url
	 * @param boolean strip_trail Will strip unwanted empty operators at the end.
	 * @return string
	 * @author Jason Schoeman
	 */
	public function buildURL ($menu_id = false, $extend_url = '', $strip_trail = '')
	{
		// Should we assign menu id.
		if (empty($menu_id)) $menu_id = $this->configuration['m'];
		// Can we use sef url?
		if (! empty($this->configuration['sef_url'])) {
			// We first need to check if we have such an alias available:
			if (empty($this->navigation["$menu_id"]['alias'])) {
				$alias = $this->db->invokeQuery('NAVIGATION_findAliasQuery', $menu_id);;
			} else {
				$alias = $this->navigation["$menu_id"]['alias'];
			}
			// Add extra string. Thought not 100% correct, this is for backwards compatibility.
			if (! empty($extend_url)) {
				$extend_url = "?$extend_url";
			} else if (! empty($strip_trail)) {
				$extend_url = false;
			} else {
				$extend_url = '?';
			}
			$url_append = empty($this->configuration['url_append']) ? '' : $this->configuration['url_append'];
			// Menu id is set for sef mode, so return correct integer id.
			$url = $alias . $url_append . "$extend_url";
		} else {
			// Add extra string.
			if (! empty($extend_url)) {
				$extend_url = "&$extend_url";
			} else {
				$extend_url = false;
			}
			// Menu id is an integer.
			$url = 'index.php?m=' . "$menu_id" . "$extend_url";
		}
		if (! empty($url)) {
			return $this->configuration['absolute_url'] . "/$url";
		} else {
			return false;
		}
	}

	/**
	 * Parses the REQUEST_URI to get the page id
	 *
	 * @date 20101007 (v1.0.2) (greg) moved from PHPDS to PHPDS_navigation ; little cleanup
	 * @date 20100109
	 * @version 1.0.2
	 * @author Ross Kuyper
	 */
	public function parseRequestString()
	{
		$configuration = $this->configuration;
		// let's set up a default page
		if ($this->user->isLoggedIn()) {
			$configuration['m'] = $configuration['front_page_id_in'];
		} else {
			$configuration['m'] = $configuration['front_page_id'];
		}
		if (! empty($_GET['m'])) {
			$configuration['m'] = $_GET['m'];
			$get_menu_id = $_GET['m'];
		} else {
			$get_menu_id = NULL;
		}
		
		// Give SEF rewrites priority if active
		if(! empty($configuration['sef_url'])) {

			// if the request string is empty, we're on the frontpage
			if(! empty($_SERVER['REQUEST_URI'])) {

				// Find everything before ?
				list($req) = explode('?', $_SERVER['REQUEST_URI']);

				// Replace base patch dir.
				$basepath = parse_url($configuration['absolute_url'], PHP_URL_PATH);

				// Destroy base paths.
				$req = trim(str_replace($basepath, '', $req),'/');

				// Make sure we have correct value.
				$uriarray = explode('/', $req);

				// Final alias should be cool now...
				$alias = array_shift($uriarray);

				// All of the rest of $uriarray should be variables.
				foreach ($uriarray as $get) {
					// First one is key.
					if (empty($key)) {
						$key = $get;
					} else {
						$getarray[$key] = $get;
						$key = '';
					}
				}

				// Assign variables to get.
				if (! empty($getarray)) {
					if(! empty($_GET)) $_GET = array_merge($getarray, $_GET);
					else $_GET = $getarray;
				}

				if (!empty($alias) && ($alias != 'index.php')) {
					// Is there a set URL append?
					if (!empty($configuration['url_append']) ) $alias = str_replace($configuration['url_append'], '', $alias);

					if (isset($this->navAlias[$alias])) {
						$configuration['m'] = $this->navAlias[$alias];
					} else {
						// Check if we can locate alias in database.
						$required_menu_id = $this->db->invokeQuery('NAVIGATION_findMenuQuery', $alias, $get_menu_id);
						if (empty($required_menu_id)) {
							// This means, the url really does not exist, lets just forward user to 404.
							//throw new PHPDS_exception(___('Page not found'), 404);
							$this->template->stopScript = array('type'=>'404','message'=>___('Page not found'));
						} else {
							if ($this->user->isLoggedIn()) {
								$this->template->stopScript = array('type'=>'403','message'=>___('Page found, however you dont have the required permission to access this page.'));
								//throw new PHPDS_securityException403(___('Page found, however you dont have the required permission to access this page.'), 403);
							} else {
								$this->template->stopScript = array('type'=>'auth','message'=>___('Authentication Required'));
								//$this->template->stopScript = ___('Authentication Required');
								$configuration['m'] = $required_menu_id;
							}
						}
					}
				}
			}
		}

		return $this;
	}

	/**
	 * This method saves the current URL with the option to add more $this->security->get variables like ("&variable1=1&variable2=2")
	 * This is mostly used for when additional $this->security->get variables are required! Usefull when using forms.
	 *
	 * @param string Add more $this->security->get variables like ("&variable1=1&variable2=2")
	 * @return string
	 * @author Jason Schoeman
	 */
	public function selfUrl ($extra_get_variables = false)
	{
		// Workaround.
		return $this->buildURL(false, $extra_get_variables, true);
	}

	/**
	 * Will convert any given plugin script location to its correct url.
	 *
	 * @param $file_path The full file path, "DummyPlugin/sample/sample1.php"
	 * @param $extend_url Should the url be extended with $_GET vars, 'e=12'
	 * @param $strip_trail Will strip unwanted empty operators at the end.
	 * @return string
	 */
	public function purl ($file_path, $extend_url = false, $strip_trail = false)
	{
		// Menu id.
		$menu_id = $this->createMenuId($file_path);

		// Get page url.
		return $this->buildURL($menu_id, $extend_url, $strip_trail);
	}

	/**
	 * Convert plugin file location to unsigned CRC32 value. This is unique and allows one to locate a menu item from location as well.
	 *
	 * @param string The plugin folder the file is in.
	 * @return integer
	 * @author Jason Schoeman
	 */
	public function createMenuId ($path)
	{
		// Create menu id from string.
		return sprintf('%u', crc32(str_ireplace('/', '', $path)));
	}

	/**
	 * Redirects to new url.
	 *
	 * @param string URL to redirect to.
	 * @param integer Time in seconds before redirecting.
	 * @author Jason Schoeman
	 */
	public function redirect ($url = false, $time = 0)
	{
		// Check if we only need to redirect to the same page, a simple refresh.
		if ($url == false) {
			$redirect_url = '<META HTTP-EQUIV="refresh" CONTENT="' . $time . '; URL=' . $this->buildURL($this->configuration['m']) . '">';
		} else {
			$redirect_url = '<META HTTP-EQUIV="refresh" CONTENT="' . $time . '; URL=' . $url . '">';
		}
		print $redirect_url;
	}

	/**
	 * Returns the url of the 404 page selected by the admin.
	 *
	 * @return string
	 */
	public function pageNotFound ()
	{
		// Menu id.
		$menu_id = $this->db->getSettings(array('404_error_page'), 'PHPDevShell');

		// Get page url.
		return $this->buildURL($menu_id['404_error_page']);
	}
}