<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 */
class PHPDS_template extends template
{
	/**
	 * Adds content to head of page.
	 * @var string
	 */
	public $modifyHead = '';
	/**
	 * Modify Output Text Logo
	 * @var mixed
	 */
	public $modifyOutputTextLogo = false;
	/**
	 * Modify Output Logo
	 * @var mixed
	 */
	public $modifyOutputLogo = false;
	/**
	 * Modify Output Time
	 * @var mixed
	 */
	public $modifyOutputTime = false;
	/**
	 * Modify Output Login Link.
	 * @var mixed
	 */
	public $modifyOutputLoginLink = false;
	/**
	 * Modify Output User.
	 * @var mixed
	 */
	public $modifyOutputUser = false;
	/**
	 * Modify Output Role.
	 * @var mixed
	 */
	public $modifyOutputRole = false;
	/**
	 * Modify Output Group.
	 * @var mixed
	 */
	public $modifyOutputGroup = false;
	/**
	 * Modify Output Title.
	 * @var mixed
	 */
	public $modifyOutputTitle = false;
	/**
	 * Modify Output Menu.
	 * @var mixed
	 */
	public $modifyOutputMenu = false;
	/**
	 * Modify Output Breadcrumbs.
	 * @var mixed
	 */
	public $modifyOutputBreadcrumbs = false;
	/**
	 * Modify Output Footer.
	 * @var mixed
	 */
	public $modifyOutputFooter = false;
	/**
	 * Modify Output Controller.
	 * @var mixed
	 */
	public $modifyOutputController = false;
	/**
	 * Modify Output Controller content.
	 * @var string
	 */
	public $controller;
	/**
	 * This variable is used to activate a stop script command, it will be used to end a script immediately while still finishing compiling the template.
	 *
	 * Usage Example :
	 * <code>
	 * // This wil make the script stop (not PHPDevShell) while still finishing the template to the end.
	 * $template->stopScript = array('type'=>'auth','message'=>'The script stopped cause I wanted it to.');
	 * </code>
	 *
	 * @var array
	 */
	public $stopScript;
	/**
	 * Assign this variable if you would like to debug your script, it will show on critical errors as well.
	 * It will parse anything that is assigned to it.
	 *
	 * @var string
	 */
	public $debugStr = false;
	/**
	 * Use this to have global available variables throughout scripts. For instance in hooks.
	 *
	 * @var array
	 */
	public $global;
	/**
	 * Use this to have global available variables throughout scripts. For instance in hooks.
	 *
	 * @var array
	 */
	public $skipLogin = false;
	/**
	 * Contains module html sniplets.
	 *
	 * @var array
	 */
	public $module = array();

	/**
	 * Name of the Theme to use (previously known as template)
	 * @since v3.1.0
	 * @var string
	 */
	public $templateName;

	/**
	 * Sends a message to login form.
	 * @var string
	 */
	public $loginMessage;

	/**
	 * Repository of data pieces to be used by the actual Theme code
	 * @since v3.1.0
	 * @var array
	 */
	protected $pieces;

	/**
	 * Content Distribution Network.
	 * If you are running a very large site, you might want to consider running a dedicated light http server (httpdlight, nginx) that
	 * only serves static content like images and static files, call it a CDN if you like.
	 * By adding a host here 'http://192.34.22.33/project/cdn', all images etc, of PHPDevShell will be loaded from this address.
	 * @var string
	 */
	protected $CDN;

	/**
	 * Run default, custom or no template.
	 *
	 * @version 1.2
	 * @date	 20100520	(v1.1) (greg) added merging with modules from the configuration array
	 * @date 20110308 (v1.2) greg) allows the new style controller to alter the current template to be used
	 * @author Jason Schoeman
	 */
	public function runTemplate ()
	{
		$configuration = $this->configuration;
		if (empty($this->templateName))
			$this->templateName = 'theme.php';

		if (empty($configuration['static_content_host'])) {
			$this->CDN = $this->configuration['absolute_url'];
		} else {
			$this->CDN = $this->configuration['static_content_host'];
		}


		// Create preferred template dir.
		$template_dir = 'themes/'.$configuration['template_folder'].'/';

		// Check if we have a module file.
		if (!$this->core->loadFile($template_dir . 'modules.php')) {
			// Load modules file.
			$this->core->loadFile('themes/cloud/modules.php');
		}
		// allow override and additions in the configuration file
		if (!empty($configuration['modules']) && is_array($configuration['modules'])) {
			$this->module = array_merge($this->module, $configuration['modules']);
		}

		try {
			ob_start();

			// old-style compatibility: either we have a new style controller (which should be executed now)
			// or the the template will execute the script at the next step
			if (empty($this->module['bc']) || ($this->module['bc'] != 2)) {
				// This will pre-load the controller and save it.
				$this->executeController();
			} else {
				$this->core->loadFile($template_dir.'language/core.lang.php');
			}

			if (empty($this->controller)) {
				$this->controller = ob_get_clean();
			} else {
				PU_cleanBuffers();
			}
		} catch (Exception $e) {
			PU_cleanBuffers();
			if (is_a($e, 'PHPDS_securityException')) {
				$this->templateName = 'login.php';
			} else if (is_a($e, 'PHPDS_pageException404')) {
				$this->templateName = '404.php';
			} else if (is_a($e, 'PHPDS_securityException403')) {
				$this->templateName = '403.php';
			} else throw $e;
		}

		if (! empty($this->templateName)) {
			// There should be a template available here...
			if ($this->core->loadFile($template_dir.$this->templateName) !== false) { return; }
			if ($this->core->loadFile($template_dir.'main.php') !== false) { return; }

			throw new PHPDS_exception('Fix me, I can\'t find the custom template!');
		} elseif (is_string($this->controller)) {
			print $this->controller;
		}
	}

	/**
	 * Will add any css path to the <head></head> tags of your document.
	 *
	 * @param string $cssRelativePath
	 */
	public function addCssFileToHead ($cssRelativePath = '', $media='screen, projection') {
		$this->modifyHead .= '<link rel="stylesheet" href="' . $this->CDN . '/' . $cssRelativePath . '" media="' . $media . '" />';
	}

	/**
	 * Will add any js path to the <head></head> tags of your document.
	 *
	 * @param string $jsRelativePath
	 */
	public function addJsFileToHead ($jsRelativePath = '') {
		$this->modifyHead .= '<script type="text/javascript" src="' . $this->CDN . '/' . $jsRelativePath . '"></script>';
	}

	/**
	 * Will add any content to the <head></head> tags of your document.
	 *
	 * @param string $extraHead
	 */
	public function addToHead ($extraHead = '') {
		$this->modifyHead .= $extraHead;
	}

	/**
	 * Will add any js to the <head></head> tags of your document adding script tags.
	 *
	 * @param string $js
	 */
	public function addJsToHead ($js = '') {
		$this->modifyHead .= '<script type="text/javascript">' . $js . '</script>';
	}

	/**
	 * Will add any css to the <head></head> tags of your document adding script tags.
	 *
	 * @param string $css
	 */
	public function addCSSToHead ($css = '') {
		$this->modifyHead .= '<style>' . $css . '</style>';
	}

	/**
	 * Changes head output.
	 * @param boolean $return
	 * @return string
	 */
	public function outputHead ($return = false) {
		// Check if we should return or print.
		if ($return == false) {
			// Simply output charset.
			print $this->modifyHead;
		} else {
			return $this->modifyHead;
		}
	}

	/**
	 * Outputs current language identifier being used.
	 *
	 * @author Jason Schoeman
	 */
	public function outputLanguage ($return = false)
	{
		// Check if we should return or print.
		if ($return == false) {
			// Simply output charset.
			print $this->configuration['language'];
		} else {
			return $this->configuration['language'];
		}
	}

	/**
	 * Outputs charset.
	 *
	 * @author Jason Schoeman
	 */
	public function outputCharset ($return = false)
	{
		// Check if we should return or print.
		if ($return == false) {
			// Simply output charset.
			print $this->configuration['charset'];
		} else {
			return $this->configuration['charset'];
		}
	}

	/**
	 * Outputs the active scripts title.
	 *
	 * @author Jason Schoeman
	 */
	public function outputTitle ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputTitle == false) {
			$navigation = $this->navigation->navigation;
			if (isset($navigation[$this->configuration['m']]['menu_name'])) {
				print $navigation[$this->configuration['m']]['menu_name'] . ' - ' . $this->configuration['scripts_name_version'];
			} else {
				print $this->stopScript['message'];
			}
		} else {
			print $this->modifyOutputTitle;
		}
	}

	/**
	 * This returns/prints the skin for inside theme usage.
	 *
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Skin.
	 * @author Jason Schoeman
	 */
	public function outputSkin ($return = 'print')
	{
		// Create HTML.
		$html = $this->configuration['skin'];

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints the absolute url for inside theme usage.
	 *
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Absolute url.
	 * @author Jason Schoeman
	 */
	public function outputAbsoluteURL ($return = 'print')
	{
		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $this->CDN;
		} else if ($return === 'return' || $return == true) {
			return $this->CDN;
		}
	}

	/**
	 * This returns/prints the meta keywords for inside theme usage.
	 *
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Meta Keywords.
	 * @author Jason Schoeman
	 */
	public function outputMetaKeywords ($return = 'print')
	{
		// Create HTML.
		$html = $this->configuration['meta_keywords'];

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints the meta description for inside theme usage.
	 *
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Meta Description.
	 * @author Jason Schoeman
	 */
	public function outputMetaDescription ($return = 'print')
	{
		// Create HTML.
		$html = $this->configuration['meta_description'];

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * Print a string via a module and returns the result
	 *
	 *
	 * @param string $moduleName the module to use
	 * @param string $string the string to print (several strings can be provided)
	 * @date 20100920 (v1.0) (greg) created
	 * @version 1.0
	 * @author greg
	 */
	public function modPrint($moduleName) // more arguments can be provided
	{
		$strings = func_get_args();
		$dep = array_shift($strings); // get rid of module name
		$result = empty($this->module[$moduleName]) ? implode($strings) : PU_sprintfn($this->module[$moduleName], $strings);
		return $result;
	}

		/**
	 * Gets the desired logo and displays it. This method will try its best to deliver a logo, whatever the case.
	 *
	 * @author Jason Schoeman
	 */
	public function outputLogo ()
	{
		if ($this->modifyOutputLogo == false) {
			// First we need to see if we will be using the custom logo.
			if (! empty($this->configuration['custom_logo'])) {
				// Give him his custom logo.
				$logo = $this->modPrint('logo', $this->CDN . '/', $this->CDN . '/' . $this->configuration['custom_logo'], $this->configuration['scripts_name_version'], $this->configuration['scripts_name_version']);
			} else {
				// Ok so we have no set logo, does the developer want a custom logo?
				if (! empty($this->db->pluginLogo)) {
						// Ok lets get the logo that the user wishes to display.
						$logo = $this->modPrint('logo', $this->CDN . '/', "{$this->CDN}/plugins/{$this->db->pluginLogo}/images/logo.png", $this->configuration['scripts_name_version'], $this->configuration['scripts_name_version']);
				} else if (! empty($this->configuration['scripts_name_version'])) {
						$logo = '<h1>' . $this->configuration['scripts_name_version'] . '</h1>';
				} else {
						// Oops we have no logo, so lets just default to the orginal PHPDevShell logo.
						$logo = $this->modPrint('logo', $this->CDN . '/', "{$this->CDN}/plugins/PHPDevShell/images/logo.png", $this->configuration['scripts_name_version'], $this->configuration['scripts_name_version']);
				}
			}
			// Ok return the logo.
			print $logo;
		} else {
			print $this->modifyOutputLogo;
		}
	}

	/**
	 * Acquire script identification image or logo.
	 *
	 * @param string $menu_link
	 * @param string $active_plugin
	 */
	public function scriptLogo ($menu_link, $active_plugin, $alias = null)
	{
		// Find last occurance.
		$filename_from = strrchr($menu_link, '/');
		if (empty($filename_from)) $filename_from = $menu_link;
		// Set image name.
		$image_name = ltrim($this->core->rightTrim($filename_from, '.php'), '/');
		// Create image url.
		$img_url_alias =!empty($alias) ?  "plugins/$active_plugin/images/$alias.png": '';
		$img_url = "plugins/$active_plugin/images/$image_name.png";
		$image_url_plugin_default = "plugins/$active_plugin/images/default.png";
		// Lets check if image exists, if not, we need to set it to use default.
		if ($img_url_alias && file_exists($img_url_alias)) {
			return $this->CDN . '/' . $img_url_alias;
		} elseif (file_exists($img_url)) {
			return $this->CDN . '/' . $img_url;
		} elseif (file_exists($image_url_plugin_default)) {
			return $this->CDN . '/' . $image_url_plugin_default;
		} else {
			return $this->CDN . '/plugins/PHPDevShell/images/default.png';
		}
	}

	/**
	 * Sets template time.
	 *
	 * @author Jason Schoeman
	 */
	public function outputTime ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputTime == false) {
			// Output active info.
			print $this->core->formatTimeDate($this->configuration['time']);
		} else {
			print $this->modifyOutputTime;
		}
	}

	/**
	 * Sets template login link.
	 *
	 * @author Jason Schoeman
	 */
	public function outputLoginLink ()
	{
		$navigation = $this->navigation;
		$configuration = $this->configuration;

		// Check if output should be modified.
		if ($this->modifyOutputLoginLink == false) {
			if ($this->user->isLoggedIn()) {
				$login_information = sprintf('<a href="%s"><div id="logged-in" class="loginlink"><span></span>%s</div></a>', $navigation->buildURL($configuration['loginandout'], 'logout=1'), $configuration['user_display_name']);
			} else {
				$login_information = sprintf('<a href="%s"><div id="logged-out" class="loginlink"><span></span>' . $navigation->navigation[$configuration['loginandout']]['menu_name'] . '</div></a>', $navigation->buildURL($configuration['loginandout']));
			}
			// Output active info.
			print $login_information;
		} else {
			print $this->modifyOutputLoginLink;
		}
	}

	/**
	 * Sets template role.
	 *
	 * @author Jason Schoeman
	 */
	public function outputRole ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputRole == false) {
			// Set active role.
			$active_role = '';
			if ($this->user->isLoggedIn())
				$active_role = sprintf(___('%s Role&#44;&nbsp;'), $this->configuration['user_role_name']);
			// Output active info.
			print $active_role;
		} else {
			print $this->modifyOutputRole;
		}
	}

	/**
	 * Sets template group.
	 *
	 * @author Jason Schoeman
	 */
	public function outputGroup ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputGroup == false) {
			// Set active role.
			$active_group = '';
			if ($this->user->isLoggedIn())
				$active_group = sprintf(___('%s Group&#44;&nbsp;'), $this->configuration['user_group_name']);
			// Output active info.
			print $active_group;
		} else {
			print $this->modifyOutputGroup;
		}
	}

	/**
	 * This returns/prints an image of the current script running.
	 *
	 * @param boolean Default is false, if set true, the heading will return instead of print.
	 * @return string Returns image tag with image url.
	 * @author Jason Schoeman
	 */
	public function outputScriptIcon ($return = false)
	{
		$navigation = $this->navigation->navigation;
		// Create script logo ////////////////////////////////////////////////////////////////////////////
		if (! empty($navigation[$this->configuration['m']]['menu_id'])) {
			$script_logo_url = $this->scriptLogo($navigation[$this->configuration['m']]['menu_link'], $navigation[$this->configuration['m']]['plugin']);
			//////////////////////////////////////////////////////////////////////////////////////////////////
			$menu_name = $navigation[$this->configuration['m']]['menu_name'];
			// Create HTML.
			$html = $this->modPrint('scriptIcon', $script_logo_url, $menu_name);
			// Return or print to browser.
			if ($return == false) {
				print $html;
			} else if ($return == true) {
				return $html;
			}
		} else {
			return false;
		}
	}

	/**
	 * Returns "breadcrumbs" to the template system. Intended to be used by the engine.
	 *
	 * @author Jason Schoeman
	 */
	public function outputBreadcrumbs ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputBreadcrumbs == false) {
			print $this->navigation->createBreadcrumbs();
		} else {
			print $this->modifyOutputBreadcrumbs;
		}
	}

	/**
	 * Returns "menus" to the template system. Intended to be used by the engine.
	 *
	 * @author Jason Schoeman
	 */
	public function outputMenu ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputMenu == false) {
			print $this->navigation->createMenuStructure();
		} else {
			print $this->modifyOutputMenu;
		}
	}

	/**
	 * Returns "output script" to the template system. Intended to be used by the engine.
	 *
	 * @author Jason Schoeman
	 */
	public function outputScript ()
	{
		$this->outputController();
	}

	/**
	 * Returns "output script" to the template system. Intended to be used by the engine.
	 *
	 * @author Jason Schoeman
	 */
	public function outputController ()
	{
		if ($this->modifyOutputController == false) {
			print $this->controller;
		} else {
			print $this->modifyOutputController;
		}
	}

	/**
	 * Sets template system logo or name.
	 *
	 * @author Jason Schoeman
	 */
	public function outputTextLogo ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputTextLogo == false) {
			// Output active info.
			print $this->configuration['scripts_name_version'];
		} else {
			print $this->modifyOutputTextLogo;
		}
	}

		/**
	 * Returns the last footer string to the template system. Intended to be used by the engine.
	 *
	 * @author Jason Schoeman
	 */
	public function outputFooter ()
	{
		// Check if output should be modified.
		if ($this->modifyOutputFooter == false) {
			print $this->configuration['footer_notes'];
		} else {
			print $this->modifyOutputFooter;
		}
	}

	/**
	 * This returns/prints a heading discription of the script being executed. Intended to be used by the developer.
	 *
	 * @version 1.1
	 *
	 * @date 20110309 (v1.1) (greg) changed to use the pieces repository
	 * @date 20110309 (v1.2) (jason) good idea but it wont work as heading is not mandatory in controllers.
	 *
	 * @param string This is the message that will be displayed as the heading.
	 * @return nothing
	 * @author Jason Schoeman
	 */
	public function heading ($heading, $return = 'print')
	{
		$html = $this->modPrint('heading', $heading);
		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints info of the script being executed. Intended to be used by the developer.
	 *
	 * @version 1.1
	 *
	 * @date 20110309 (v1.1) (greg) changed to use the pieces repository
	 * @date 20110309 (v1.2) (jason) good idea but it wont work as info is not mandatory in controllers.
	 *
	 * @param string This is the message that will be displayed as the info.
	 * @return nothing
	 * @author Jason Schoeman
	 */
	public function info ($information, $return = 'print')
	{
		// Create HTML.
		$html = $this->modPrint('info', $information);
		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This method will load given png icon from icon database,
	 *
	 * @param string Icon name without extention.
	 * @param Title of given image.
	 * @param int The size folder to look within.
	 * @param string If an alternative class must be added to image.
	 * @param string File type.
	 * @param boolean Default is false, if set true, the heading will return instead of print.
	 */
	public function icon($name, $title=false, $size=16, $class='icon', $type='.png', $return=true)
	{
		$navigation = $this->navigation->navigation;
		// Create icon dir.
		$script_url = $this->CDN . '/themes/' . $navigation[$this->configuration['m']]['template_folder'] . '/images/icons-' . $size . '/' . $name . $type;
		if (empty ($title))
			$title = '';
		// Create HTML.
		$html = $this->modPrint('icon', $script_url, $title, $class);

		// Return or print to browser.
		if ($return == false) {
			print $html;
		} else if ($return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a warning message regarding the active script. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @param mixed default is log, can be set true, print, return.
	 * @return string Warning string.
	 * @author Jason Schoeman
	 */
	public function warning ($warning, $return = 'print', $log = 'log')
	{
		if ($log === true || $log == 'log') {
			// Log types are : ////////////////
			// 1 = OK /////////////////////////
			// 2 = Warning ////////////////////
			// 3 = Critical ///////////////////
			// 4 = Log-in /////////////////////
			// 5 = Log-out ////////////////////
			///////////////////////////////////
			$log_type = 2; ////////////////////
			// Log the event //////////////////
			$this->db->logArray[] = array('log_type' => $log_type , 'log_description' => $warning);
		}
		// Create HTML.
		$html = $this->modPrint('warning', $warning);
		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a ok message regarding the active script. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @param mixed default is log, can be set true, print, return.
	 * @return string Ok string.
	 * @author Jason Schoeman
	 */
	public function ok ($ok, $return = 'print', $log = 'log')
	{
		if ($log === true || $log == 'log') {
			// Log types are : ////////////////
			// 1 = OK /////////////////////////
			// 2 = Warning ////////////////////
			// 3 = Critical ///////////////////
			// 4 = Log-in /////////////////////
			// 5 = Log-out ////////////////////
			///////////////////////////////////
			$log_type = 1; ////////////////////
			// Log the event //////////////////
			$this->db->logArray[] = array('log_type' => $log_type , 'log_description' => $ok);
		}
		// Create HTML.
		$html = $this->modPrint('ok', $ok);
		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a error message regarding the active script. Intended to be used by the developer where exceptions are caught.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @param mixed default is log, can be set true, print, return.
	 * @return string Error string.
	 * @author Jason Schoeman
	 */
	public function error ($error, $return = 'print', $log = 'log')
	{
		if ($log === true || $log == 'log') {
			// Log types are : ////////////////
			// 1 = OK /////////////////////////
			// 2 = Warning ////////////////////
			// 3 = Critical ///////////////////
			// 4 = Log-in /////////////////////
			// 5 = Log-out ////////////////////
			// 6 = Error //////////////////////
			///////////////////////////////////
			$log_type = 6; ////////////////////
			// Log the event //////////////////
			$this->db->logArray[] = array('log_type' => $log_type , 'log_description' => $error);
		}
		// Create HTML.
		$html = $this->modPrint('error', $error);
		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a critical message regarding the active script. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @param mixed default is log, can be set true, print, return.
	 * @return string Critical string.
	 * @author Jason Schoeman
	 */
	public function critical ($critical, $return = 'print', $log = 'log', $mail = 'mailadmin')
	{
		$navigation = $this->navigation->navigation;
		if ($log === true || $log == 'log') {
			// Log types are : ////////////////
			// 1 = OK /////////////////////////
			// 2 = Warning ////////////////////
			// 3 = Critical ///////////////////
			// 4 = Log-in /////////////////////
			// 5 = Log-out ////////////////////
			///////////////////////////////////
			$log_type = 3; ////////////////////
			// Log the event //////////////////
			$this->db->logArray[] = array('log_type' => $log_type , 'log_description' => $critical);
		}
		// Check if we need to email admin.
		if ($this->configuration['email_critical']) {
			// Subject.
			$subject = sprintf(___("CRITICAL ERROR NOTIFICATION %s"), $this->configuration['scripts_name_version']);
			// Message.
			$broke_script = $navigation[$this->configuration['m']]['menu_name'];
			$broken_url = $this->configuration['absolute_url'] . '/index.php?m=' . $this->configuration['m'];
			$message = sprintf(___("Dear Admin,\r\n\r\nTHERE WAS A CRITICAL ERROR IN %s:\r\n\r\n%s\r\n\r\nClick on url to access broken script called %s:\r\n%s\r\n\r\nScript error occurred for user:\r\n%s"), $this->configuration['scripts_name_version'], $critical, $broke_script, $broken_url, $this->configuration['user_display_name']);
			if ($mail === true || $mail == 'mailadmin') {
				// Initiate email class.
				$email = $this->factory('mailer');
				// Ok we can now send the critical email message.
				$email->sendmail("{$this->configuration['setting_admin_email']}", $subject, $message);
			}
		}
		// Create HTML.
		$html = $this->modPrint('critical', $critical);

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a notice of the script being executed. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Notice string.
	 * @author Jason Schoeman
	 */
	public function notice ($notice, $return = 'print')
	{
		// Create HTML.
		$html = $this->modPrint('notice', $notice);

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a busy of the script being executed. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Busy string.
	 * @author Jason Schoeman
	 */
	public function busy ($busy, $return = 'print')
	{
		// Create HTML.
		$html = $this->modPrint('busy', $busy);

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a message of the script being executed. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Message string.
	 * @author Jason Schoeman
	 */
	public function message ($message, $return = 'print')
	{
		// Create HTML.
		$html = $this->modPrint('message', $message);

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a note of the script being executed. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Note string.
	 * @author Jason Schoeman
	 */
	public function note ($note, $return = 'print')
	{
		// Create HTML.
		$html = $this->modPrint('note', $note);

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This returns/prints a heading of the script being executed. Intended to be used by the developer.
	 *
	 * @param string This is the message that will be displayed.
	 * @param mixed default is print, can be set true, print, return.
	 * @return string Heading string.
	 * @author Jason Schoeman
	 */
	public function scripthead ($scripthead, $return = 'print')
	{
		// Create HTML.
		$html = $this->modPrint('scripthead', $scripthead);

		// Return or print to browser.
		if ($return === 'print' || $return == false) {
			print $html;
		} else if ($return === 'return' || $return == true) {
			return $html;
		}
	}

	/**
	 * This creates an the [i] when over with mouse a popup with a message appears, this can be placed anywhere. Intended to be used by the developer.
	 *
	 * @param string The message to diplay when mouse goes over the [i].
	 * @param boolean Sets to print out confirm link instead of return.
	 * @author Jason Schoeman
	 */
	public function tip ($text, $print = false)
	{
		// This is yet another IE Fix !
		$text_clean = preg_replace('/"/', '', $text);
		$info = $this->modPrint('tooltip', $text_clean);
		if ($print == false) {
			return $info;
		} else {
			print $info;
		}
	}

	/**
	 * This function support output_script by looking deeper into menu structure to find last linked menu item that is not linked to another.
	 *
	 * @param integer $extended_menu_id
	 * @return integer
	 */
	public function extendMenuLoop ($extended_menu_id)
	{
		// Assign extention value.
		$extend_more = $this->navigation->navigation[$extended_menu_id]['extend'];
		// Check if we should look higher up for a working menu id and prevent endless looping.
		if (! empty($extend_more) && ($extended_menu_id != $this->navigation->navigation[$extend_more]['extend'])) {
			$this->extendMenuLoop($extend_more);
		} else {
			// Final check, to see if we had an endless loop that still has an extention.
			if (! empty($this->navigation->navigation[$extended_menu_id]['extend'])) {
				if (! empty($this->navigation->navigation[$extended_menu_id]['parent_menu_id'])) {
					// Lets look even higher up now that we jumped the endless loop.
					$this->extendMenuLoop($this->navigation->navigation[$extended_menu_id]['parent_menu_id']);
				} else {
					// We now have no other choice but to show default home page.
					$this->extended_menu_id = 0;
				}
			} else {
				$this->extended_menu_id = $extended_menu_id;
			}
		}
	}

	/**
	 * Executes the controller.
	 *
	 * @author Jason Schoeman
	 */
	public function executeController()
	{
		$this->view = '';
		$navigation = $this->navigation->navigation;
		$configuration = $this->configuration;

		////////////////////////////////////////////////////////////////////////////////////////
		// Start SQL transactions.
		$this->db->startTransaction();
		// Are we in demo mode?
		if ($configuration['demo_mode'] == true) {
			if ($configuration['user_role'] != $configuration['root_role']) {
				// Show demo mode message for end user.
				$this->notice(sprintf(___('%s is currently in a demo mode state, no actual database transactions will occur.'), $configuration['scripts_name_version']));
			} else {
				// Show demo mode message for root user.
				$this->notice(sprintf(___('%s is currently in a demo mode state, only Root users are able to save database transactions.'), $configuration['scripts_name_version']));
			}
		}
		// Load script to buffer.
		if (! empty($navigation[$configuration['m']]['menu_id'])) {
			// We need to assign active menu_id.
			$menu_id = $configuration['m'];
			// Determine correct menu action.
			switch ($navigation[$configuration['m']]['menu_type']) {
				// Plugin File.
				case 1:
					// Do case.
					$menu_case = 1;
					break;
				// Link.
				case 2:
					break;
				// Jump.
				case 3:
					break;
				// External File.
				case 4:
					// Do case.
					$menu_case = 4;
					break;
				// HTTP URL.
				case 5:
					// Do case.
					$menu_case = 5;
					break;
				// Placeholder.
				case 6:
					break;
				// iFrame.
				case 7:
					// Do case.
					$menu_case = 7;
					break;
				// Cronjob.
				case 8:
					// Do case.
					$menu_case = 8;
					break;
				default:
					// Do case.
					$menu_case = 1;
					break;
			}
			///////////////////////////////////
			// Do further checking on links. //
			///////////////////////////////////
			if (empty($menu_case)) {
				// So we have some kind of link, we now need to see what kind of link we have.
				// Get menu extended data.
				$extend = $navigation[$menu_id]['extend'];
				// Get menu type.
				if (!empty($navigation[$extend]['menu_type'])) {
					$linked_menu_type = $navigation[$extend]['menu_type'];
				} else {
					PHPDS_exception(sprintf(___('I had a problem extending menu item %s, it does not seem to exist.'), $extend));
				}
				// We now have the linked menu type and can now work accordingly.
				// Determine correct menu action.
				switch ($linked_menu_type) {
					// Plugin File.
					case 1:
						// Do case.
						$menu_case = 1;
						// We need to assign active menu_id.
						$menu_id = $extend;
						break;
					// Link.
					case 2:
						// Do case.
						$menu_case = 2;
						// We need to assign active menu_id.
						$this->extendMenuLoop($navigation[$extend]['extend']);
						$menu_id = $this->extended_menu_id;
						break;
					// Jump.
					case 3:
						// Do case.
						$menu_case = 2;
						// We need to assign active menu_id.
						$this->extendMenuLoop($navigation[$extend]['extend']);
						$menu_id = $this->extended_menu_id;
						break;
					// External File.
					case 4:
						// Do case.
						$menu_case = 4;
						// We need to assign active menu_id.
						$menu_id = $extend;
						break;
					// HTTP URL.
					case 5:
						// Do case.
						$menu_case = 5;
						// We need to assign active menu_id.
						$menu_id = $extend;
						break;
					// Placeholder.
					case 6:
						// Do case.
						$menu_case = 2;
						// We need to assign active menu_id.
						$this->extendMenuLoop($navigation[$extend]['extend']);
						$menu_id = $this->extended_menu_id;
						break;
					// iFrame.
					case 7:
						// Do case.
						$menu_case = 7;
						// We need to assign active menu_id.
						$menu_id = $extend;
						break;
					// Cronjob.
					case 8:
						// Do case.
						$menu_case = 8;
						// We need to assign active menu_id.
						$menu_id = $extend;
						break;
					default:
						// Do case.
						$menu_case = 1;
						// We need to assign active menu_id.
						$menu_id = $extend;
						break;
				}
			}
			// Execute repeated menu cases.
			switch ($menu_case) {
				// Plugin Script.
				case 1:
					$this->loadControllerFile($menu_id);
					break;
				// Link, Jump, Placeholder.
				case 2:
					// Is this an empty menu item?
					if (empty($menu_id)) {
						// Lets take user to the front page as last option.
						// Get correct frontpage id.
						($this->user->isLoggedIn()) ? $menu_id = $configuration['front_page_id_in'] : $menu_id = $configuration['front_page_id'];
					}
					$this->loadControllerFile($menu_id);
					break;
				// External File.
				case 4:
					// Require external file.
					if (!$this->core->loadFile($navigation[$menu_id]['menu_link'])) {
						PHPDS_exception(sprintf(___('File could not be found, I tried to execute filename : %s'), $navigation[$menu_id]['menu_link']));
					}
					break;
				// HTTP URL.
				case 5:
					// Redirect to external http url.
					$this->ok(sprintf(___('You are now being redirected to an external url, %s'), $navigation[$menu_id]['menu_link']), false, false);
					$this->navigation->redirect($navigation[$menu_id]['menu_link']);
					break;
				// iFrame.
				case 7:
					// Clean up height.
					$height = preg_replace('/px/i', '', $navigation[$menu_id]['extend']);
					// Create Iframe.
					$this->HTML = $this-> modPrint('iframe', $navigation[$menu_id]['menu_link'], $height, '100%');
					break;
				// Cronjob.
				case 8:
					// Require script.
					if (!$this->loadControllerFile($menu_id)) {
						$time_now = time();
						// Update last execution.
						$this->db->invokeQuery('TEMPLATE_cronExecutionLogQuery', $time_now, $menu_id);
						// Always log manual touched cronjobs.
						$this->ok(sprintf(___('Cronjob %s executed manually.'), $navigation[$menu_id]['menu_name']));
					}
					break;
			}
			// This is used for debugging scripts.
			// It will always show if $template->debugStr is set.
			($this->debugStr) ? $this->notice($this->debugStr) : null;
			// Print HEREDOC HTML.
			if (! empty($this->HTML)) print $this->HTML;
		} else {
			$this->stopScript = array('type'=>'auth','message'=>___('Authentication Required'));
		}
		if (isset($this->stopScript)) {
			// Roll back current transaction.
			$this->db->invokeQuery('TEMPLATE_rollbackQuery');
			switch ($this->stopScript['type']) {
			case 'auth':
				throw new PHPDS_securityException($this->stopScript['message']);
			break;

			case '404':
				throw new PHPDS_pageException404($this->stopScript['message'],$this->stopScript['type']);
			break;

			case '403':
				throw new PHPDS_securityException403($this->stopScript['message'],$this->stopScript['type']);
			break;

			default:
				throw new PHPDS_securityException($this->stopScript['message'],$this->stopScript['type']);
			break;
			}
		}
		// End database transactions.
		$this->db->endTransaction();
	}

	/**
	 * Will attempt to load controller file from various locations.
	 *
	 * @version 1.0.1
	 *
	 * @date 20100917 (v1.0) (Jason)
	 * @date 20110308 (v1.0.1) (greg) loadFile returns an exact false when the file is not found
	 *
	 * @author Jason Schoeman
	 * @param int
	 */
	public function loadControllerFile ($menu_id, $include_model = false)
	{
		$navigation = $this->navigation->navigation;
		$core = $this->core;
		// Set plugin folder.

		if (!empty($navigation[$menu_id])) {
			$plugin_folder = $navigation[$menu_id]['plugin_folder'];
			// Do we want to include the model.
			if ($include_model) {
				if ($include_model === true) $include_model = 'query';
				$core->loadFile($plugin_folder . 'models/' . preg_replace("/.php/", '.' . $include_model . '.php', $navigation[$menu_id]['menu_link']));
			}
			// Set active script directory.
			$active_dir = $plugin_folder . '%s' . $navigation[$menu_id]['menu_link'];
			// Require script.
			if ($core->loadFile(sprintf($active_dir, 'controllers/')) !== false) return true;
			if ($core->loadFile(sprintf($active_dir, '')) !== false) return true;
		}
		if (empty($this->stopScript)) {
			throw new PHPDS_exception(sprintf(___('The controller of menu id %d could not be found, I tried to execute filename : "%s"'), $menu_id, sprintf($active_dir, '{controllers/}')));
		}
		return false;
	}

	/**
	 * Login heading messages.
	 *
	 * @author Jason Schoeman
	 */
	public function loginFormHeading($return = false)
	{
		$HTML = '';
		$message= '';
		if (! empty($this->loginMessage))
			$message = $this->notice(___($this->loginMessage), 'return');

		// Create headings for login.
		if (! empty($this->stopScript)) {
			$HTML .= $this->heading(___('Authentication Required'), 'return');
			$HTML .= $message;
		} else {
			// Get some default settings.
			$settings = $this->db->getSettings(array('login_message'));

			// Check if we have a login message to display.
			if (! empty($settings['login_message'])) {
				$login_message = $this->message(___($settings['login_message']), 'return');
			} else {
				$login_message = '';
			}

			$HTML .= $this->heading(___('Login'), 'return');
			$HTML .= $login_message;
			$HTML .= $message;
		}

		if ($return == false) {
			print $HTML;
		} else {
			return $HTML;
		}
	}

	/**
	 * Executes the login.
	 *
	 * @author Jason Schoeman
	 */
	public function loginForm($return = false)
	{
		$HTML = $this->factory('StandardLogin')->loginForm($return);

		if ($return == false) {
			print $HTML;
		} else {
			return $HTML;
		}
	}

	/**
	 * Get and return the supposed to run template.
	 *
	 * @return string if not found, return default.
	 * @author Jason Schoeman
	 */
	public function getTemplate ()
	{
		// Check if the menu has a defined template.
		if (! empty($this->navigation->navigation[$this->configuration['m']]['template_folder'])) {
			$settings['default_template'] = $this->navigation->navigation[$this->configuration['m']]['template_folder'];
		} else {
			// If not check if the gui system settings was set with a default template.
			$settings['default_template'] = $this->configuration['default_template'];
		}

		// Return the complete template.
		return $settings['default_template'];
	}

	/**
	 * Gets the correct location of a tpl file, will return full path, can be a view.tpl or view.tpl.php files.
	 *
	 * @param string $load_view
	 * @param string $plugin_override If another plugin is to be used in the directory.
	 */
	public function getTpl($load_view=false, $plugin_override=false)
	{
		$configuration = $this->configuration;
		$navigation = $this->navigation;
		$core = $this->core;

		// Menu link.
		if (empty($navigation->navigation[$configuration['m']]['extend'])) {
			$menu_link = $navigation->navigation[$configuration['m']]['menu_link'];
		} else {
			$menu_link = $navigation->navigation[$navigation->navigation[$configuration['m']]['extend']]['menu_link'];
			// Set plugin for this menu item.
			$plugin_extend = $navigation->navigation[$navigation->navigation[$configuration['m']]['extend']]['plugin'];
		}
		// Do template engine.
		if (empty($plugin_override) && empty($plugin_extend)) {
			$plugin_folder = $configuration['absolute_path'] . 'plugins/' . $core->activePlugin() . '/';
		} else if (! empty($plugin_override)) {
			$plugin_folder = $configuration['absolute_path'] . 'plugins/' . $plugin_override . '/';
		} else if (! empty($plugin_extend)) {
			$plugin_folder = $configuration['absolute_path'] . 'plugins/' . $plugin_extend . '/';
		}

		// Do we have a custom template file?
		if (empty($load_view) && !empty($navigation->navigation[$configuration['m']]['layout'])) {
			$load_view = $navigation->navigation[$configuration['m']]['layout'];
		}
		// Check if we have a template file otherwise use default.
		if (empty($load_view)) {
			$tpl_dir = str_replace($menu_link, '%s/' . str_replace('.php', '.tpl', $menu_link), $plugin_folder . $menu_link);
		} else {
			$link = strrchr($menu_link, '/');
			if (empty($link)) {
				$tpl_dir = $plugin_folder . '%s/' . $load_view;
			} else {
				$link = str_replace($link, '', $menu_link);
				$tpl_dir = $plugin_folder . '%s/' . $link . '/' . $load_view;
			}
		}
		// Log to firephp.
		$this->_log(__('Loading Template Layout : ', 'core.lang') . $tpl_dir);
		// Return file location.
		if (file_exists(sprintf($tpl_dir, 'views'))) {
			$tpldir = sprintf($tpl_dir, 'views');
			return $tpldir;
		// Perhaps we have a php template.
		} else if (file_exists(sprintf($tpl_dir . '.php', 'views'))) {
			return sprintf($tpl_dir . '.php', 'views');
		} else {
			return false;
		}
	}

	/**
	 * Prints the number of queries used to the gui.
	 *
	 * @param string $count_queries
	 */
	public function debugInfo ()
	{
		if (! empty($this->db->countQueries)) {
			$count_queries = $this->db->countQueries;
		} else {
			$count_queries = 0;
		}
		if ($this->configuration['queries_count']) {
			if (!empty($this->templateName)) {
				$memory_used = memory_get_peak_usage();
				$queries_used = sprintf('(Queries Used : %s) - (PHP Memory Used : %s Mb)', $count_queries, number_format($memory_used / 1000000, 2, '.', ' '));
				print $this->modPrint('queries_count', $queries_used);
			}
		}
	}

	/**
	 * Convert all HTML entities to their applicable characters.
	 *
	 * @param string $string_to_decode
	 * @return string
	 */
	public function htmlEntityDecode ($string_to_decode)
	{
		// Decode characters.
		return html_entity_decode($string_to_decode, ENT_QUOTES, $this->configuration['charset']);
	}

	/**
	 * returns a piece from the repository, optionnaly throught a module
	 *
	 * @version 1.0
	 * @since v3.1.0
	 * @author greg <greg@phpdevshell.org>
	 * @date 20110309 (v1.0) (greg) added
	 *
	 * @param $name string, the name of the piece to fetch
	 * @param $module string or boolean, (optionnal) the name of the module to use, true to use a module with the piece name, false not to use a module
	 */
	public function getPiece($name, $module = false)
	{
		$data = isset($this->pieces[$name]) ? $this->pieces[$name] : '';
		if ($module) {
			if ($module === true) {
				$module = $name;
			}
			return $this->modPrint($module, $data);
		} else {
			return $data;
		}
	}

	/**
	 * insert a piece into the repository
	 *
	 * @version 1.0
	 * @since v3.1.0
	 * @author greg <greg@phpdevshell.org>
	 * @date 20110309 (v1.0) (greg) added
	 *
	 * @param $name string, the name of the piece to add
	 * @param $data string, the raw content of the piece
	 */
	public function setPiece($name, $data = null)
	{
		$this->pieces[$name] = $data;
	}
}

/**
 * Creates a language tooltip string and prints it out to the template.
 *
 * @param string $info_mark
 */
function tip ($text)
{
	global $template;
	$template->tip(_($text), true);
}
/**
 * Creates a language tooltip string inside a text domain and prints it out to the template.
 *
 * @param string $info_mark
 */
function dtip ($text, $domain)
{
	global $template;
	$template->tip(dgettext($domain, $text), true);
}
