<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * LOGIN - Selects users persistant login details.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org, Ross Kuyper
 *
 */
class LOGIN_selectUserPersistentQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			t1.user_id, t1.user_display_name, t1.user_password, t1.user_name, t1.user_email, t1.user_group, t1.user_role, t1.language, t1.timezone as user_timezone, t1.region,
			t2.user_group_name, t3.user_role_name
		FROM
			_db_core_users t1
		LEFT JOIN
			_db_core_user_groups t2
		ON
			t1.user_group = t2.user_group_id
		LEFT JOIN
			_db_core_user_roles t3
		ON
			t1.user_role = t3.user_role_id
		WHERE
			t1.user_id = '%s'
	";
	
	protected $singleRow = true;
}

/**
 * LOGIN - Selects users cookie data.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class LOGIN_selectCookieQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_id, cookie_id, pass_crypt
		FROM
			_db_core_session
		WHERE
			id_crypt = '%s'
	";
}

/**
 * LOGIN - Deletes users cookie data.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class LOGIN_deleteCookieQuery extends PHPDS_query
{
	protected $sql = "
		DELETE FROM
			_db_core_session
		WHERE
			cookie_id = %u
	";
}

/**
 * LOGIN - Insert persistant cookie data.
 * @author Ross Kuyper
 *
 */
class LOGIN_setPersistentCookieQuery extends PHPDS_query
{
	protected $sql = "
		INSERT INTO
			_db_core_session (user_id, id_crypt, pass_crypt, timestamp)
		VALUES
			(%u, '%s', '%s', '%s')
	";

	public function invoke($parameters = null)
	{
		$user_id = $parameters[0];
		$pass_crypt = md5(uniqid(rand(), TRUE));
		$id_crypt = substr(md5(uniqid(rand(), TRUE)), 6, 6);
		$timestamp = time();

		parent::invoke(array($user_id, $id_crypt, $pass_crypt, $timestamp));

		return setcookie('pdspc', $id_crypt . $pass_crypt, $timestamp + 63113852);
	}
}

/**
 * LOGIN - Remove persistant cookie from database
 * @author Ross Kuyper
 *
 */
class LOGIN_deletePersistentCookieQuery extends PHPDS_query
{
	protected $sql = "
		DELETE FROM
			_db_core_session
		WHERE
			(id_crypt = '%s' AND user_id = '%s')
	";

	public function invoke($parameters = null)
	{
		if (!empty($_COOKIE['pdspc'])) {
			$user_id = $parameters[0];

			$id_crypt = substr($_COOKIE['pdspc'], 0, 6);

			parent::invoke(array($id_crypt, $user_id));

			return setcookie('pdspc', 'false', 0);
		}
	}
}

/**
 * LOGIN - Gets user detail from database.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class LOGIN_selectUserQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			t1.user_id, t1.user_display_name, t1.user_password, t1.user_name, t1.user_email, t1.user_group, t1.user_role, t1.language, t1.timezone as user_timezone, t1.region,
			t2.user_group_name, t3.user_role_name
		FROM
			_db_core_users t1
		LEFT JOIN
			_db_core_user_groups t2
		ON
			t1.user_group = t2.user_group_id
		LEFT JOIN
			_db_core_user_roles t3
		ON
			t1.user_role = t3.user_role_id
		WHERE
			(t1.user_name = '%(username)s' OR t1.user_email = '%(username)s')
		AND
			IF('%(password)s' = '*', 1, t1.user_password = '%(password)s')
	";
	protected $singleRow = true;

	public function checkParameters(&$parameters = null)
	{
		list($username, $password) = $parameters;
		$parameters = $this->protectArray(array('username' => $username, 'password' => $password));
		// The parameters are considered as valid.
		return true;
	}

	public function checkResults(&$results = null)
	{
		if (empty($results))
			$results = false; //  to be consistent with the original API
		return true;
	}
}

/**
 * LOGIN - Checks if username exists.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class LOGIN_selectUserNameQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			t1.user_id
		FROM
			_db_core_users t1
		WHERE
			(t1.user_name = '%(username)s' OR t1.user_email = '%(username)s')
	";
	protected $singleRow = true;

	public function checkParameters(&$parameters = null)
	{
		list($username) = $parameters;
		$parameters = $this->protectArray(array('username' => $username));
		// The parameters are considered as valid.
		return true;
	}

	public function checkResults(&$results = null)
	{
		if (empty($results))
			$results = false; //  to be consistent with the original API
		return true;
	}
}

















